##########################################################################
#
#  localisation.r
#
#  A lot of code for localisation tests in R
#
##########################################################################

library("vegan")
library("MASS")
library("shapes")


numNodes = length(bib1nodes);
dim=2;

localiseMDS <- function(distMatrix,nodesCount,nodesPos)
{
  mds <- cmdscale(as.dist(distMatrix),add=TRUE,eig=TRUE,k=nodesCount-1)
  xy <- cbind(mds$points[,1],-mds$points[,2])

  #browser()
  good <- procrustes(nodesPos, xy)
  
  #browser()
  good$Yrot
}







testMDS <- function(doplot = T,noisesd=0)
{
  dist <- function(a,b) {
    sqrt( ((noeuds.x[a]-noeuds.x[b])^2) + ((noeuds.y[a]-noeuds.y[b])^2) )
  }

  mat <- array(0, dim=c(9,9))

  noeuds.x <- c(0,5,10,0,5,10,0,5,10)
  noeuds.y <- c(0,0,0,5,5,5,10,10,10)

  noeuds.x <- noeuds.x - mean(noeuds.x)
  noeuds.y <- noeuds.y - mean(noeuds.y)

  for (i in 1:9) {
    for (j in 1:9) {
      if (i != j) {
        mat[i,j] <- dist(i,j)
      }
    }
  }

  mat <- mat + rnorm(81,0,noisesd)

  diag(mat) <- 0

  plotab <- localiseMDS(mat,9,cbind(noeuds.x,noeuds.y))

  if (doplot) {
    x11()
    plot(noeuds.x,noeuds.y,main='test MDS localisation no error in distance data',col="blue1",pch="o")
    points(plotab,col="green1",pch="+")
  }
  sse <- 0
  for (i in 1:length(noeuds.x)) {
    sse <- sse + ( (noeuds.x[i]-plotab[i,1])^2 + (noeuds.y[i]-plotab[i,2])^2 )
  }

  sqrt(sse/length(noeuds.x))
}


#
# WARNING !!! LATERATION FAILS IF 3 ANCHORS ARE PERFECTLY ALIGNED
#
#
testLAT <- function(doplot = T,noisesd=0.4,ac=3) {
	dist <- function(a,b) {
	sqrt( ((noeuds.x[a]-noeuds.x[b])^2) + ((noeuds.y[a]-noeuds.y[b])^2) )
	}

	mat <- array(0, dim=c(9,9))
	 noeuds.x <- c(0,5,10,0,5,10,0,5,10)
	  noeuds.y <- c(0,0,0,5,5,5,10,10,10)
	
	  noeuds.x <- noeuds.x - mean(noeuds.x)
	  noeuds.y <- noeuds.y - mean(noeuds.y)
	
	  for (i in 1:9) {
	    for (j in 1:9) {
	      if (i != j) {
	        mat[i,j] <- dist(i,j)
	      }
	    }
  	}

	good<-F

	while(!good) {

	#print("dd")

	choos <- sample(1:9,ac,F)

	#check if not degenerated
	xa<-T
	ya<-T
	for (k in 1:(ac-1)) {
		if ( (noeuds.x[choos[k]] == noeuds.x[choos[k+1]]) ) {
			xa <- xa & T
		} else {
			xa <- xa & F
		}

		if ( (noeuds.x[choos[k]] == noeuds.x[choos[k+1]]) ) {
			ya <- ya & T
		} else {
			ya <- ya & F
		}
	}
	if (xa==T | ya==T) { good<-F } 
	else { good<-T }

	}


	nchoos <- numeric()

	cnt<-1
	for (k in 1:9) {
		if ( all( k != choos ) ) {
			nchoos[cnt] <- k
			cnt<-cnt+1
		}
	}

	mat <- mat + rnorm(81,0,noisesd)

	#localiseLATAO <- function(distMatrix,nodesIndex,anchorsPos,anchorsIndex)#
	plotab <- localiseLATAO(mat,nchoos,cbind(noeuds.x[choos],noeuds.y[choos]),choos)



	if (doplot) {
		x11()
		plot(noeuds.x,noeuds.y,main='test LATAO localisation no error in distance data',col="blue1",pch="o")
		points(plotab,col="green1",pch="+")
	}

	sse <- 0
	for (w in 1:(length(nchoos))) {
	sse <- sse +  ((noeuds.x[nchoos[w]]-plotab[w,1])^2) + ((noeuds.y[nchoos[w]]-plotab[w,2])^2)
	}


	sqrt(sse/length(nchoos))
}

testStabilityNoise <- function(doit = 10) {

  erseq <- seq(.05,5,by=.05)
  erMDS <- numeric()
  erLAT <- numeric()
  for (i in 1:length(erseq)) {
	erMDS[i] <- 0
	erLAT[i] <- 0
  }
  pdf("testStabilityNoise.pdf")
  plot(NULL,NULL,type='n',xlim=c(0,5),ylim=c(0,5))

  for (i in 1:length(erseq)) {
  	for (j in 1:doit) {
  	    erMDS[i] <- erMDS[i] + testMDS(F,erseq[i])

    	}
  }
  
  erMDS <- erMDS / doit
  points(erseq,erMDS,type="l")
  
  for (k in 3:8) {
	for (i in 1:length(erseq)) {
  		for (j in 1:doit) {
  		    erLAT[i] <- erLAT[i] + testLAT(F,erseq[i],k)
  		}
	}
	erLAT <- erLAT / doit

	points(erseq,erLAT,type="l",col=hsv((k-2)/6,0.8,1))

	for (i in 1:length(erseq)) {
		erLAT[i] <- 0
	}
    	
  }

  legend(x = 3.5, y = 1, col=c("black",hsv((1:6)/6,0.8,1)) ,lty = 1,legend=c("MDS",paste("LAT avec",(3:8),"ancres")),cex=1/2)
  
  dev.off()
 
 
}



testLatFctNumAnchorsTheo<-function() {

	
	
	dist <- function(a,b) {
	sqrt( ((real.x[a]-real.x[b])^2) + ((real.y[a]-real.y[b])^2) )
	}
	
	numnod<-length(bib1nodes)
	noisesd<-2
	nbTests<-1000
  
	noeuds.x <- real.x[bib1nodes-1]
	noeuds.y <- real.y[bib1nodes-1]

	noeuds.x <- noeuds.x - mean(noeuds.x)
	noeuds.y <- noeuds.y - mean(noeuds.y)

	mati <- array(0, dim=c(numnod,numnod))
	mat <- array(0, dim=c(numnod,numnod))
	
	for (i in 1:numnod) {
	    for (j in 1:numnod) {
	      if (i != j) {
		mati[i,j] <- dist(i,j)
	      }
	    }
	  }
	  
	diag(mati) <- 0	  
	
	
	
	
	nn <- 3:16
	
	
	pdf("testLatFctConnectivity.pdf")
	plot(NULL,NULL,xlab="Nombre d'ancres",ylab="Erreur localisation",type="n",xlim=c(3,16),ylim=c(0,15))
	
	maxkk = 1.4
	kkseq <-seq(0.6,maxkk,.2)
	
	for(kk in kkseq) {
	
		rmse<-numeric()

		for (i in 1:(length(nn))) {

			tmp<-numeric()

			for (j in 1:nbTests) {

				
				good<-F

				while(!good) {

					#print("dd")

					choos <- sample(1:18,nn[i],F)

					#check if not degenerated
					xa<-T
					ya<-T
					for (k in 1:(nn[i]-1)) {
						if ( (real.x[bib1nodes[choos[k]]-1] == real.x[bib1nodes[choos[k+1]]-1]) ) {
							xa <- xa & T
						} else {
							xa <- xa & F
						}

						if ( (real.y[bib1nodes[choos[k]]-1] == real.y[bib1nodes[choos[k+1]]-1]) ) {
							ya <- ya & T
						} else {
							ya <- ya & F
						}
					}
					if (xa==T | ya==T) { good<-F } 
					else { good<-T }

				}


				nchoos <- numeric()

				cnt<-1
				for (k in 1:numnod) {
					if ( all( k != choos ) ) {
						nchoos[cnt] <- k
						cnt<-cnt+1
					}
				}
				
				mat <- mati + rnorm(18*18,0,kk)

				#localiseLATAO <- function(distMatrix,nodesIndex,anchorsPos,anchorsIndex)#
				plotab <- localiseLATAO(mat,nchoos,cbind(noeuds.x[choos],noeuds.y[choos]),choos)
				sse <- 0

				for (w in 1:(length(nchoos))) {
					sse <- sse +  ((noeuds.x[nchoos[w]]-plotab[w,1])^2) + ((noeuds.y[nchoos[w]]-plotab[w,2])^2)
				}

				tmp[j] <- sqrt(sse/length(nchoos))

			}

			rmse[i]<- mean(tmp)

		}
	
	
		points(nn,rmse,col=hsv(kk/maxkk,0.8,1),type="l")
	
	
	}
	
	
	legend(x = 7, y = 15, col=hsv(kkseq/maxkk,0.8,1) ,lty = 1,legend=kkseq)
	
	dev.off()

}


testLatFctNumAnchorsBib1<-function() {
	nbTests<-1000
	numnod<-length(bib1nodes)
	
	nn <- 3:16
	
	#get a model for dist = f(lqi)
	lin <- reg1(1000,F)


	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib1()

	#convert to distance following model	
	mat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)
	
	pdf("testLatFctConnectivityBib1.pdf")
	plot(NULL,NULL,xlab="Nombre d'ancres",ylab="Erreur localisation",type="n",xlim=c(3,16),ylim=c(0,15))
	
	
	rmse<-numeric()
	
	#print("aa")

	for (i in 1:(length(nn))) {
	
		#print("bb")

		tmp<-numeric()

		for (j in 1:nbTests) {
		
			#print("cc")
			
			good<-F
			
			while(!good) {
			
				#print("dd")
				
				choos <- sample(1:18,nn[i],F)
				
				#check if not degenerated
				xa<-T
				ya<-T
				for (k in 1:(nn[i]-1)) {
					if ( (real.x[bib1nodes[choos[k]]-1] == real.x[bib1nodes[choos[k+1]]-1]) ) {
						xa <- xa & T
					} else {
						xa <- xa & F
					}
					
					if ( (real.y[bib1nodes[choos[k]]-1] == real.y[bib1nodes[choos[k+1]]-1]) ) {
						ya <- ya & T
					} else {
						ya <- ya & F
					}
				}
				if (xa==T | ya==T) { good<-F } 
				else { good<-T }
				
			}
			
			
			nchoos <- numeric()

			cnt<-1
			for (k in 1:numnod) {
				if ( all( k != choos ) ) {
					nchoos[cnt] <- k
					cnt<-cnt+1
				}
			}
			

			#localiseLATAO <- function(distMatrix,nodesIndex,anchorsPos,anchorsIndex)#
			plotab <- localiseLATAO(mat,nchoos,cbind(real.x[bib1nodes[choos]-1],real.y[bib1nodes[choos]-1]),choos)
			sse <- 0

			for (w in 1:(length(nchoos))) {
				sse <- sse +  ((real.x[bib1nodes[nchoos[w]]-1]-plotab[w,1])^2) + ((real.y[bib1nodes[nchoos[w]]-1]-plotab[w,2])^2)
			}

			tmp[j] <- sqrt(sse/length(nchoos))

		}

		rmse[i]<- mean(tmp)

	}


	points(nn,rmse,col="red",type="l")
	
	dev.off()
	
	
}



getLqisBib1 <-function(i,j) {
  bib1$lqi[bib1$src==i & bib1$pongsrc==j]
}

getLqiMeanMatrixBib1<-function() {
  mm <- array(dim=c(length(bib1nodes),length(bib1nodes)))
  diag(mm) <- 0
  for (i in 1:length(bib1nodes)) {
    for (j in 1:length(bib1nodes)) {
      if (i!=j) {
        mm[i,j] <- mean(getLqisBib1(bib1nodes[i],bib1nodes[j]))
      }
    }
  }
  mm
}

getLqisBib2 <-function(i,j,dataslqis,datassrc,dataspong) {
  dataslqis[datassrc==i & dataspong==j]
}

getLqiMeanMatrixBib2 <- function(radf) {
	sel <- ((bib2$seqno%/%10)+1)==radf
	dataslqis <- bib2$lqi[sel]
	datassrc <- bib2$src[sel]	
	dataspong <- bib2$pongsrc[sel]	

 mm <- array(dim=c(length(bib1nodes),length(bib1nodes)))
  diag(mm) <- 0
  for (i in 1:length(bib1nodes)) {
    for (j in 1:length(bib1nodes)) {
      if (i!=j) {
        mm[i,j] <- mean(getLqisBib2(bib1nodes[i],bib1nodes[j],dataslqis,datassrc,dataspong))
      }
    }
  }
	#browser()
  mm
}



getRealDistMatrixBib1<-function() {
 mm <- array(dim=c(length(bib1nodes),length(bib1nodes)))
  diag(mm) <- 0
  for (i in 1:length(bib1nodes)) {
    for (j in 1:length(bib1nodes)) {
      if (i!=j) {
        mm[i,j] <-distance(bib1nodes[i],bib1nodes[j])
      }
    }
  }
  mm
}

distance <- function(i,j) {
  sqrt((real.x[i-1]-real.x[j-1])^2+(real.y[i-1]-real.y[j-1])^2)
}


# simple mds on biblio1 datas
# use xpdistance outside datas to calibrate regression
mdsSimple1<-function() {
      lqisss <- getLqiMeanMatrixBib1()

      dm <- getLqiDistMatOutside(31)

      lqis <- numeric()
      dists <- numeric()
      cnt <- 0

      for (i in 1:length(dm)) {
        for(j in 1:length(dm[[i]])) {
          cnt <- cnt+1
          dists[cnt] <- i
          lqis[cnt] <- dm[[i]][j]
        }
      }

      mod <- lm(dists ~ lqis)

      print(min(lqis))
      print(max(lqis))
      print(min(lqisss))
      print(max(lqisss))
      
      #comp.dist <- mod$coefficients[1] + mod$coefficients[2]*lqisss
      comp.dist <- (-10.50084 + (lqisss * 0.09973))
      diag(comp.dist) <- 0
      
      #comp.dist <- comp.dist - min(comp.dist)
      
      #reflect
      
      for (i in 2:length(bib1nodes)) {
        for (j in 1:(i-1)) {
          #comp.dist[i,j] <- 0
          #comp.dist[j,i] <- 0
        }
      }
      
      diag(comp.dist) <- 0
      
      #browser()
      
      mds <- cmdscale(comp.dist,add=TRUE,eig=TRUE,k=17)
      x <- mds$points[,1]
      y <- -mds$points[,2]

      x<- x-min(x)
      y<-y-min(y)

      #center on origin
      #x <- x - mean(x)
      #y <- y - mean(y)

      #rotation et scaling des positions rsultantes pour s'adapter au "bon plan"

      xy <- cbind(x,y)
      xoyo <-cbind(real.x[bib1nodes],real.y[bib1nodes])

      browser()

      good <- procrustes(xoyo, xy, scale=T)

      plot(good)
      browser()
}


localizeMDSREGLQI <- function(doplot=T) {
  inp1 <- bib1


  mat <- array(0, dim=c(19,19))
  matnoise <- array(0, dim=c(19,19))
  mat2 <- array(0, dim=c(19,19))

  diag(mat) <- 0

  lqis <- numeric()

  len = 10000 #length(inp1$src)

  for (i in 2:19) {
    for (j in (i+1):20) {
        x <- 0
        for (k in 1:len) {
          if ((inp1$src[k] == i && inp1$pongsrc[k] == j) || ( inp1$src[k] == j && inp1$pongsrc[k] == i)) {
            lqis[x] <- inp1$lqi[k]
            x<-x+1
          }
        }
        mat[j-1,i-1] <- (-10.50084 + (mean(lqis) * 0.09973)) #to get dissimilarities


    }


  }

  mds <- cmdscale(as.dist(mat),add=TRUE,eig=TRUE,k=18)
  print("Eigen values MDS LQI")
  print(mds$eig)

  x <- mds$points[,1]
  y <- -mds$points[,2]

  xy <- cbind(x,y)
  xoyo <-cbind(real.x,real.y)



  good <- procrustes(xoyo, xy)

  mse <-  sqrt(good$ss/19)

  plotab <- good$Yrot

  if (doplot) {
    x11()
    plot(good,main='MDS localisation with DIST = F(LQI)',sub=paste('position MSE = ',mse))
  }
  mse
}



localizeMDSREGLQINOISE <- function(loops=100,noise=1,doplot=T) {
 inp1 <- bib1

  mat <- array(0, dim=c(19,19))
  matnoise <- array(0, dim=c(19,19))

  diag(mat) <- 0
  lqis <- numeric()

  len = 10000 #length(inp1$src)

  for (i in 2:19) {
    for (j in (i+1):20) {
        x <- 0
        for (k in 1:len) {
          if ((inp1$src[k] == i && inp1$pongsrc[k] == j) || ( inp1$src[k] == j && inp1$pongsrc[k] == i)) {
            lqis[x] <- inp1$lqi[k]
            x<-x+1
          }
        }
        mat[j-1,i-1] <- (-10.50084 + (mean(lqis) * 0.09973)) #to get dissimilarities
    }
  }

 x<-as.numeric(matrix(0,19,1))
 y<-as.numeric(matrix(0,19,1))

  for (i in 1:loops){
    matnoise <- (-10.50084 + (mat * 0.09973)) + rnorm(19*19,0,noise)
    diag(matnoise)<-0
    for (i in 2:19) {
    for (j in (i+1):20) {
        matnoise[i-1,j-1]<-0
    }}

    #browser()

    mds <- cmdscale(as.dist(matnoise),add=TRUE,eig=TRUE,k=18)

    x <- x + mds$points[,1]
    y <- y - mds$points[,2]

    #browser()
  }

  x <- x / loops
  y <- y / loops

  xy <- cbind(x,y)
  xoyo <-cbind(real.x,real.y)

  good <- procrustes(xoyo, xy)

  rmse <-  sqrt(good$ss/19)

  plotab <- good$Yrot

  if (doplot) {
    x11()
    plot(good,main=paste('MDS localisation with DIST = F(LQI) + NOISE : ',noise,' * ',loops),sub=paste('position RMSE = ',rmse))
  }
  rmse
}

testLocalizeMDSREGLQINOISE <- function(times = 10,loops = 1000,noise = 1) {

  rmse<-numeric()
  for (i in 1:times) {
    rmse[i]<-localizeMDSREGLQINOISE(loops,noise,doplot=F)
  }
  postscript(paste("MDSNOISE_test_times",times,"_loops_",loops,"_noise_",noise,".ps"))
  plot(1:times,rmse,type='l')
  abline(h=localizeMDSREGLQI(F),col="red1")
  dev.off()
}

testNOISE<-function(loops=1000,noise=seq(0.1,2,by=.1)) {
  rmse<-numeric()
  for (i in 1:length(noise)) {
    rmse[i]<-localizeMDSREGLQINOISE(loops,noise[i],doplot=F)
  }
  postscript(paste("MDSNOISEINC_test_loops_",loops,"_noisef_",noise[1],"_noiset_",noise[length(noise)],".ps"))
  plot(noise,rmse,type='l')
  abline(h=localizeMDSREGLQI(F),col="red1")
  dev.off()
}


localizeMDSREGLQIBESTEIG <- function() {
  inp1 <- bib1

  # cmdscale(d, k = 2, eig = FALSE, add = FALSE, x.ret = FALSE)
  # as.dist(m, diag = FALSE, upper = FALSE)
  # dissimMatrix <- matrix with lower triangle = dissimilarities

 	ji <- inp1$lqi > 128
	inp1$lqi[ji] = 383 - inp1$lqi[ji]
	inp1$lqi[!ji] = 128 - inp1$lqi[!ji]

  mat <- array(0, dim=c(19,19))
  matnoise <- array(0, dim=c(19,19))
  mat2 <- array(0, dim=c(19,19))

  diag(mat) <- 0

  lqis <- numeric()

  len = 10000 #length(inp1$src)

  for (i in 2:19) {
    for (j in (i+1):20) {
        x <- 0
        for (k in 1:len) {
          if ((inp1$src[k] == i && inp1$pongsrc[k] == j) || ( inp1$src[k] == j && inp1$pongsrc[k] == i)) {
            lqis[x] <- inp1$lqi[k]
            x<-x+1
          }
        }
        mat[j-1,i-1] <- (-10.50084 + (mean(lqis) * 0.09973)) #to get dissimilarities
        #mat[j-1,i-1] <- mod$coefficients[1] + mod$coefficients[2]*mean(lqis)

    }


  }


 loops <- 1000
 noise <- 1

 bestmds <- cmdscale(as.dist(mat),add=TRUE,eig=TRUE,k=18)

  for (i in 1:loops){
    matnoise <- (-10.50084 + (mat * 0.09973)) + rnorm(19*19,0,noise)
    diag(matnoise)<-0
    for (i in 2:19) {
    for (j in (i+1):20) {
        matnoise[i-1,j-1]<-0
    }}

    #browser()

    mds <- cmdscale(as.dist(matnoise),add=TRUE,eig=TRUE,k=18)

    if (isBetterMdsEig(mds$eig,bestmds$eig)) {
      bestmds <- mds
    }

    #browser()
  }

    x <- bestmds$points[,1]
    y <- bestmds$points[,2]
 
  #x <- x / loops
  #y <- y / loops


  #x <- x - mean(x)
  #y <- y - mean(y)




  xy <- cbind(x,y)
  xoyo <-cbind(real.x,real.y)



  good <- procrustes(xoyo, xy)

  rmse <-  sqrt(good$ss/19)

  plotab <- good$Yrot

  #browser()
  x11()
  plot(good,main=paste('MDS localisation with DIST = F(LQI) + NOISE : ',noise,' * ',loops),sub=paste('position RMSE = ',rmse))
  #plot (plotab,col="blue",main="localisatio MDS experience NO5",xlim=c(-3,3),ylim=c(-3,3)) #,
  #text (plotab,labels=2:20,col="blue")


  #points (xoyo,col="red")
  #text (xoyo,labels=2:20,col="red")

  browser()
}

isBetterMdsEig<- function(e1, e2) {
  v1 <- e1[1]*e1[2]*e1[3] / e1[4]*e1[5]*e1[6]
  v2 <- e2[1]*e2[2]*e2[3] / e2[4]*e2[5]*e2[6]
  print(v1)
  print(v2)

  (v1 > v2)
}

addNoise <- function(distM,sd=1) {
	sd <- 1

	distM+rnorm(numNodes*numNodes,0,sd)
}


#1 test multiprocrustes.

getModelBib1 <- function(doplot=F) {
	lqis <- numeric()
	dists <- numeric()

	len <- 10000
	for (i in 1:len) {
	  if (bib1$src[i] > 1 & bib1$pongsrc[i] > 1) {
	    lqis[i] <- bib1$lqi[i]
	    dists[i] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
	  }
	}

	lin=lm(dists ~ lqis)

	#browser()

	if (doplot) {
		pdf("model_lqi_dist_bib1_simple.pdf")
		plot(lqis,dists,xlab="lqis",
		ylab="dists",
		main="Dists-Lqi simple reg",
		sub=paste("rmse=",sqrt(mean(lin$residuals^2))))
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
lin
}

getModelBib1Rssi <- function(doplot=F) {
	lqis <- numeric()
	dists <- numeric()

	len <- 10000
	for (i in 1:len) {
	  if (bib1$src[i] > 1 & bib1$pongsrc[i] > 1) {
	    lqis[i] <- bib1$rssi[i]
	    dists[i] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
	  }
	}

	lin=lm(dists ~ lqis)

	#browser()

	if (doplot) {
		pdf("model_lqi_dist_bib1_simple.pdf")
		plot(lqis,dists,xlab="lqis",
		ylab="dists",
		main="Dists-Lqi simple reg",
		sub=paste("rmse=",sqrt(mean(lin$residuals^2))))
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
lin
}


getModelBib2 <- function(doplot=F,radf=15) {
	lqis <- numeric()
	dists <- numeric()

	cnt <- 1
 	for (i in 1:length(bib2$idx)) {
    aa <- bib2$seqno[i] %/% 10
    if ((aa+1) == radf) {
			if (bib2$src[i] > 1 & bib2$pongsrc[i] > 1) {	
				lqis[cnt] <- bib2$lqi[i]
				dists[cnt] <- distanceBib1(bib2$src[i],bib2$pongsrc[i])
				cnt <- cnt+1
			}
		}
  }



	lin=lm(dists ~ lqis)

	#browser()

	if (doplot) {
		pdf(paste("model_lqi_dist_bib2_simple_",radf,".pdf"))
		plot(lqis,dists,xlab="lqis",
		ylab="dists",
		main=paste("Dists-Lqi simple reg radf=",radf),
		sub=paste("rmse=",sqrt(mean(lin$residuals^2))))
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
lin
}



getModelExpBib1 <- function(doplot=F,lqimax=110) {
	lqis <- numeric()
	dists <- numeric()

	len <- 10000
	for (i in 1:len) {
	  if (bib1$src[i] > 1 & bib1$pongsrc[i] > 1) {
	    lqis[i] <- bib1$lqi[i]
	    dists[i] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
	  }
	}

	df <- data.frame(cbind(lqis,dists))
	lqis <- lqis / max(lqis)

	lin <- lm (dists ~ exp(lqis))

	browser()

	if (doplot) {
		pdf("model_exp_lqi_dist_bib1_simple.pdf")
	
		
		plot(lqis,dists,xlab="lqis",
		ylab="dists",
		main="Dists-Lqi simple reg",
		sub=paste("rmse=",sqrt(mean(lin$residuals^2))))

		a <- seq(60,110,1)
		
		b <- lin$coefficients[1] + (lin$coefficients[2] * exp(a/max(a)))
		points(a, b ,type="l",col="red")
	}
	
lin
}


test_MDSBGPA <- function(doplot=T,BAG.cnt = 100,noisesd=3) {
	#get a model for dist = f(lqi)
	lin <- getModelBib1()
	
	#get the lqi mean matrix from bib 1
	distMat <- getLqiMeanMatrixBib1()

	#convert lqi to dist from model
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)


	posArr <- array(dim=c(numNodes,dim,BAG.cnt))
  
	# for BAG.cnt times 
	for (i in 1:BAG.cnt) {
		#add some noise
		noisyDistMat <- addNoise(distMat,noisesd)

		#compute MDS
		mdsTmp <- cmdscale(as.dist(noisyDistMat),add=TRUE,eig=FALSE,k=numNodes-1)
			x <- mdsTmp$points[,1]
    	y <- mdsTmp$points[,2]


		#save
		posArr[,,i] <- cbind(x,y)   
		
	}

	#do mixing with all positions matrix
	#posArray is a (points * dim * samples)  array
	
	gpa <- procGPA(posArr)

	x <- 1:numNodes
	y <- 1:numNodes

	
	#average positions of all rotated datas.
	for (j in 1:numNodes) {
		x[j] <-	mean(gpa$rotated[j,1,])
		y[j] <- mean(gpa$rotated[j,2,])			
	}

	#fit on real for displaying error
 	xy <- cbind(x,y)
  xoyo <-cbind(real.x[bib1nodes[1:numNodes]-1],real.y[bib1nodes[1:numNodes]-1])
  good <- procrustes(xoyo, xy) 
  rmse <-  sqrt(good$ss/numNodes)

	if (doplot) {
		plot(good,main=paste('MDS'),sub=paste('position RMSE = ',rmse))
	}

	good
}

test_MDSBIB2 <- function(doplot=T,radf=15) {

	#get a model for dist = f(lqi)
	lin <- getModelBib2(radf)


	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib2(radf)

	#convert to distance following model	
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)

	#apply MDS
	mdsTmp <- cmdscale(as.dist(distMat),add=TRUE,eig=FALSE,numNodes-1)
			x <- mdsTmp$points[,1]
    	y <- mdsTmp$points[,2]

	#fit on real for displaying error
 	xy <- cbind(x,y)
  xoyo <-cbind(real.x[bib1nodes[1:numNodes]-1],real.y[bib1nodes[1:numNodes]-1])
  good <- procrustes(xoyo, xy) 
  rmse <-  sqrt(good$ss/numNodes)

	if (doplot) {
		plot(good,main=paste('MDS'),sub=paste('position RMSE = ',rmse))
	}
	
	good

}

test_MDS <- function(doplot=T) {

	#get a model for dist = f(lqi)
	lin <- getModelBib1()


	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib1()

	#convert to distance following model	
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)

	#apply MDS
	mdsTmp <- cmdscale(as.dist(distMat),add=TRUE,eig=FALSE,numNodes-1)
			x <- mdsTmp$points[,1]
    	y <- mdsTmp$points[,2]

	#fit on real for displaying error
 	xy <- cbind(x,y)
  xoyo <-cbind(real.x[bib1nodes[1:numNodes]-1],real.y[bib1nodes[1:numNodes]-1])
  good <- procrustes(xoyo, xy) 
  rmse <-  sqrt(good$ss/numNodes)

	if (doplot) {
		plot(good,main=paste('MDS'),sub=paste('position RMSE = ',rmse))
	}
	
	good
}

# WARNING !!! LATERATION FAILS IF 3 ANCHORS ARE PERFECTLY ALIGNED
#localise by lateration calculating with base anchor only at each time
localiseLATAO <- function(distMatrix,nodesIndex,anchorsPos,anchorsIndex)
{
	m <- length(anchorsIndex)
	n <- length(nodesIndex)
  found.x <- numeric()
  found.y <- numeric()


  anchors.x <- anchorsPos[,1]
  anchors.y <- anchorsPos[,2]

  #found.x[1:m] <- anchorsPos[1:m,1]
  #found.y[1:m] <- anchorsPos[1:m,2]

  for(i in 1:n) {
    A <- matrix(data=NA,m-1,2)
    b <- matrix(data=NA,m-1,1)
    distAnchorsPoints <- numeric()

    distAnchorsPoints <- distMatrix[anchorsIndex,nodesIndex[i]]

		#browser()

    for (j in 1:(m-1)) {
      A[j,1] <- 2 * (anchors.x[j] - anchors.x[m])
      A[j,2] <- 2 * (anchors.y[j] - anchors.y[m])
      b[j] <- (anchors.x[j]^2) - (anchors.x[m]^2) + (anchors.y[j]^2) - (anchors.y[m]^2) + distAnchorsPoints[m]^2 - distAnchorsPoints[j]^2
    }
    aaa <- numeric()
    #try(aaa <- solve(A, b))
    aaa <-  ginv(t(A) %*% A) %*% t(A) %*%  b

    found.x[i] <- aaa[1,1]
    found.y[i] <- aaa[2,1]
  }
  cbind(found.x,found.y)
}



test_LATBIB1 <- function() {

	#let's use 4 anchors
	anchors = c(1,4,16,17)
	nodes = c(2,3,5:15,18)
	
	#get a model for dist = f(lqi)
	lin <- getModelBib1()

	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib1()

	#convert to distance following model	
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)

	xy <- localiseLATAO(distMat, 							#dist matrix
		nodes, 																	#nodes index
		cbind(real.x[anchors],real.y[anchors]), #anchors pos
		anchors) 																#anchors cnt

	xoyo <-cbind(real.x[bib1nodes[nodes]-1],real.y[bib1nodes[nodes]-1])
  rmse <-  sqrt( mean((xy-xoyo)^2) )
	x11()
	plot(xy,col='blue',xlim=c(-10,10),ylim=c(-10,10),main=paste('LAT SIMPLE'),sub=paste('position RMSE = ',rmse))
	points(real.x,real.y,col='red',pch="+")


	browser()
		
}

test_LAT_DIFF_ANCHORS <- function() {
	for (x in 1:13) {


	}
}

test_LAT_ANCHORS_COUNT <- function() {
	#let's use 4 anchors to start
	anchors = c(1,4,16,17)
	nodes = c(2,3,5:15,18)
	#get a model for dist = f(lqi)
	lin <- getModelBib1()

	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib1()

	#convert to distance following model	
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)

	for (x in 1:13) {

		xy <- localiseLATAO(distMat, 							#dist matrix
			nodes, 																	#nodes index
			cbind(real.x[anchors],real.y[anchors]), #anchors pos
			anchors) 																#anchors cnt		
		
		
		anchors = cbind(anchors,nodes[1])
		nodes = nodes[2:length(nodes)]
			
	}


}

test_LAT_BAGGING <- function(doplot=T,BAG.cnt = 100,noisesd=1) {
	#let's use 4 anchors
	anchors = c(1,4,16,17)
	nodes = c(2,3,5:15,18)

	#get a model for dist = f(lqi)
	lin <- getModelBib1()

	#get LQI mean matrix from bib1
	distMat <- getLqiMeanMatrixBib1()

	#convert to distance following model	
	distMat <- lin$coefficients[1] + (lin$coefficients[2] * distMat)

	

	posArr <- array(dim=c(numNodes,dim,BAG.cnt))
  
	# for BAG.cnt times 
	for (i in 1:BAG.cnt) {
		#add some noise
		noisyDistMat <- addNoise(distMat,noisesd)

		xy <- localiseLATAO(noisyDistMat, 							#dist matrix
		nodes, 																	#nodes index
		cbind(real.x[anchors],real.y[anchors]), #anchors pos
		anchors) 																#anchors cnt)



		#save
		posArr[,,i] <- xy  
		
	}


	x <- 1:numNodes
	y <- 1:numNodes

	
	#average positions of all rotated datas.
	for (j in 1:numNodes) {
		x[j] <-	mean(posArr[j,1,])
		y[j] <- mean(posArr[j,2,])			
	}


	xy <- cbind(x,y)
  xoyo <-cbind(real.x[bib1nodes[1:numNodes]-1],real.y[bib1nodes[1:numNodes]-1])
  rmse <-  sqrt( mean((xy-xoyo)^2) )

	if (doplot) {
		x11()
		plot(xy,main=paste('MDS'),sub=paste('position RMSE = ',rmse),col='blue')
		points(xoyo,col='red')
	}

	good


}

test_MDSsstability <- function() {
	testLength <- 100
	valsBib1MdsBGPA1 <- numeric()
	valsBib1MdsBGPA2 <- numeric()
	valsBib1MdsBGPA3 <- numeric()

	#valsBib2MdsBGPA <- numeric()

#	valsMds <- numeric()
#	valsLat <- numeric()

	pdf("test_mds_stability.pdf")
	for (i in 1:testLength) {
		valsBib1MdsBGPA1[i] <- sqrt(test_MDSBGPA(F,100,1)$ss/numNodes)
		valsBib1MdsBGPA2[i] <- sqrt(test_MDSBGPA(F,100,2)$ss/numNodes)
		valsBib1MdsBGPA3[i] <- sqrt(test_MDSBGPA(F,100,3)$ss/numNodes)
		#valsBib2MdsBGPA[i] <- sqrt(test_MDSBGPA(F,100,4)$ss/numNodes)

#		valsMds[i] <- sqrt(test_MDS()$ss/numNodes)

	#	valsLat[i] <- test_LATBIB1(19,nodesIndex,anchorsPos,anchorsIndex,anchorsCount)
	}

	#plot(valsBib1MdsBGPA1,type="l",col="red",ylim=c(0,3))
	#points(valsBib1MdsBGPA2,type="l",col="blue")
	#points(valsBib1MdsBGPA3,type="l",col="green")

	dev.off()

}




testMDSBagingDiffNoise <- function() {
	testLength <- 10
	valsBib1MdsBGPA1 <- numeric()
	valsBib1MdsBGPA2 <- numeric()
	valsBib1MdsBGPA3 <- numeric()

	pdf("bib1_mds_bagging_diff_noise.pdf")
	for (i in 1:testLength) {
		valsBib1MdsBGPA1[i] <- sqrt(test_MDSBGPA(F,100,1)$ss/numNodes)
		valsBib1MdsBGPA2[i] <- sqrt(test_MDSBGPA(F,100,3)$ss/numNodes)
		valsBib1MdsBGPA3[i] <- sqrt(test_MDSBGPA(F,100,5)$ss/numNodes)
	}

	#plot(valsBib1MdsBGPA1,type="l",col="red",ylim=c(0,3),
	#	xlab="Test index",
	#	ylab="Rmse localisation",
	#	main="Mds Bagging BIB1 differents noises (Bag.CNT=100)")

	#points(valsBib1MdsBGPA2,type="l",col="blue")

	#points(valsBib1MdsBGPA3,type="l",col="green")

	#legend("topleft",
	#	legend=c(paste("sd=1 rmse mean=",round(mean(valsBib1MdsBGPA1),3)," var=",round(var(valsBib1MdsBGPA1),3))
	#	,paste("sd=3 rmse mean=",round(mean(valsBib1MdsBGPA2),3)," var=",round(var(valsBib1MdsBGPA2),3))
	#	,paste("sd=5 rmse mean=",round(mean(valsBib1MdsBGPA3),3)," var=",round(var(valsBib1MdsBGPA3),3))),
	#	col=c("red","blue","green"),
	#	pch = "-", lty = 1)
	
	aaa <- data.frame(cbind(valsBib1MdsBGPA1,valsBib1MdsBGPA2,valsBib1MdsBGPA3))
	boxplot(aaa)

	dev.off()
}

testMDSBagingDiffBagCnt <- function() {
	testLength <- 10
	valsBib1MdsBGPA1 <- numeric()
	valsBib1MdsBGPA2 <- numeric()
	valsBib1MdsBGPA3 <- numeric()

	pdf("bib1_mds_bagging_diff_bagcnt.pdf")
	for (i in 1:testLength) {
		valsBib1MdsBGPA1[i] <- sqrt(test_MDSBGPA(F,10,3)$ss/numNodes)
		valsBib1MdsBGPA2[i] <- sqrt(test_MDSBGPA(F,100,3)$ss/numNodes)
		valsBib1MdsBGPA3[i] <- sqrt(test_MDSBGPA(F,1000,3)$ss/numNodes)
	}

	#plot(valsBib1MdsBGPA1,type="l",col="red",ylim=c(0,3),
	#	xlab="Test index",
	#	ylab="Rmse localisation",
	#	main="Mds Bagging BIB1 differents Bag.CNT")

	#points(valsBib1MdsBGPA2,type="l",col="blue")
	
	#points(valsBib1MdsBGPA3,type="l",col="green")

	#legend("topleft",
	#legend=c(paste("Bag.Cnt=10 rmse mean=",round(mean(valsBib1MdsBGPA1),3)," var=",round(var(valsBib1MdsBGPA1),3))
	#	,paste("Bag.Cnt=100 rmse mean=",round(mean(valsBib1MdsBGPA2),3)," var=",round(var(valsBib1MdsBGPA2),3))
	#	,paste("Bag.Cnt=1000 rmse mean=",round(mean(valsBib1MdsBGPA3),3)," var=",round(var(valsBib1MdsBGPA3),3))),
	#	col=c("red","blue","green"),
	#	pch = "-", lty = 1)
	
	aaa <- data.frame(cbind(valsBib1MdsBGPA1,valsBib1MdsBGPA2,valsBib1MdsBGPA3))
	boxplot(aaa)

	dev.off()
}





test_MDSCONN<-function() {

}





### CODE USED IN MEMOIRE ###

#Simple Linear Regression on LQI
#Take max values at random from BIB1 data
#do a regression print to a pdf
#print rmse

reg1<-function(maxs = 100,doplot=T) {

	vals <- numeric()
	dists <- numeric()
	
	cnt<-1
	
	rMax = length(bib1$lqi)

	i<-0
	
	while (cnt < maxs) {
		i<-runif(1,1,rMax)
		
		for (j in 1:length(bib1nodes)) {
			for (k in 1:length(bib1nodes)) {
				if(bib1nodes[j]==bib1$src[i] & bib1nodes[k]==bib1$pongsrc[i]) {
					vals[cnt] <- bib1$lqi[i]
					dists[cnt] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
					cnt<-cnt+1
				}
			}
		}
	}
	
	lin <- lm(dists ~ vals)
	
	
	
	if (doplot==T) {
		rmse = sqrt(mean(lin$residuals^2))
		print(paste("rmse=",rmse))
		
		pdf(paste("reg1",maxs,".pdf"))
		plot(vals,dists,
			pch="+",
			col="blue",
			ylab="Distance (m)",
			xlab="Valeur de LQI")
		
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
	lin

}

reg1r<-function(maxs = 100,doplot=T) {

	vals <- numeric()
	dists <- numeric()
	
	cnt<-1
	
	rMax = length(bib1$rssi)

	i<-0
	
	while (cnt < maxs) {
		i<-runif(1,1,rMax)
		
		for (j in 1:length(bib1nodes)) {
			for (k in 1:length(bib1nodes)) {
				if(bib1nodes[j]==bib1$src[i] & bib1nodes[k]==bib1$pongsrc[i]) {
					vals[cnt] <- bib1$rssi[i]
					dists[cnt] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
					cnt<-cnt+1
				}
			}
		}
	}
	
	
	lin <- lm(dists ~ vals)
	
	
	
	if (doplot==T) {
		rmse = sqrt(mean(lin$residuals^2))
		print(paste("rmse=",rmse))
		
		pdf(paste("reg1_rssi",maxs,".pdf"))
		plot(vals,dists,
			pch="+",
			col="blue",
			ylab="Distance (m)",
			xlab="Valeur de RSSI",
			xlim=c(-12,0))
		
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
	lin

}

reg1rlog<-function(maxs = 100,doplot=T) {

	vals <- numeric()
	dists <- numeric()
	
	cnt<-1
	
	rMax = length(bib1$lqi)

	i<-0
	
	while (cnt < maxs) {
		i<-runif(1,1,rMax)
		
		for (j in 1:length(bib1nodes)) {
			for (k in 1:length(bib1nodes)) {
				if(bib1nodes[j]==bib1$src[i] & bib1nodes[k]==bib1$pongsrc[i]) {
					vals[cnt] <- bib1$rssi[i]
					dists[cnt] <- distanceBib1(bib1$src[i],bib1$pongsrc[i])
					cnt<-cnt+1
				}
			}
		}
	}
	md <- max(dists)
	
	vals <- 10^(-6-vals/100)
	
	lin <- lm(dists ~ vals)
	
	
	
	if (doplot==T) {
		rmse = sqrt(mean(lin$residuals^2))
		print(paste("rmse=",rmse))
		
		pdf(paste("reg1_rssi_log",maxs,".pdf"))
		plot(vals,dists,
			pch="+",
			col="blue",
			ylab="Distance (m)",
			xlab="Valeur de RSSI")
		
		abline(lin$coefficients,col="red")
		dev.off()
	}
	
	lin

}

testStabReg1<-function(nbMesures=c(10,50,100,500,1000) ){
	nbTests<-100
	
	pdf("testStabReg1.pdf")
	
	
	valsss <- array(dim=c(nbTests,length(nbMesures)))
	
	for (i in 1:length(nbMesures)) {
	
		for (j in 1:nbTests) {
			valsss[j,i]<-sqrt(mean(reg1(nbMesures[i],F)$residuals^2))	
		}
		

		
	}
	aaa <- data.frame(valsss)
	
	browser()
	
	boxplot(aaa)
	
	#legend(x = 1, y = 2, col=hsv((1:length(nbMesures))/length(nbMesures),.8,1) ,lty = 1,legend = nbMesures)
	
	dev.off()

}

testPvalueReg1<-function() {
	nbTests<-10
	s <- seq(5,100,5)

	pdf("testPvalueReg1.pdf")
	plot(NULL,type="n",xlab="Nombre de donnes dans calibration",ylab="P value",xlim=c(5,100),ylim=c(0,0.2))
	
	
	vals1<-numeric()	
	vals2<-numeric()	
	
	for (j in 1:length(s)) {
		tmp1 <- numeric()	
		tmp2 <- numeric()	
		
		for (i in 1:nbTests) {
			lin <- reg1(s[j],F)
			tmp1[i] <- summary(lin)$coefficients[1,4]
			tmp2[i] <- summary(lin)$coefficients[2,4]
		}
		
		vals1[j]<-mean(tmp1)
		vals2[j]<-mean(tmp2)
	}
	points(s,vals1,col="blue",type="l")
	points(s,vals2,col="red",type="l")
	
	legend(x = 1, y = 2, col=c("blue","red"),lty = 1,legend = c("Coef a","Coef b"))
	
	
	dev.off()	


}
