/*
 * SerialReceiver.java
 *
 * Created on 8 mars 2007, 16:57
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package net.tinyos.mapviewer;


import net.tinyos.tools.NodeInfoMsg;
import net.tinyos.tools.CalibrationMsg;

import net.tinyos.util.*;
import net.tinyos.message.*;

import java.util.LinkedList;
import java.util.ListIterator;

import java.awt.*;


/**
 *
 * @author dwarfy
 * This components listen on serial
 * When a NodeInfoMsg arrives it puts it at the start of the nodeInfoList
 * When a CalibrationMsg arrives it puts it at the start of the nodeInfoList
 */
public class SerialReceiver implements MessageListener {
    private MoteIF mote;
    private LinkedList<NodeInfoMsg> nodeInfoList;
    private LinkedList<CalibrationMsg> calibrationList;
    private MapContainerPanel M;
    private RcvPanel RP;

    public void messageReceived(int dest_addr, Message msg) {
    if (msg instanceof NodeInfoMsg) {
      nodeInfoReceived( dest_addr, (NodeInfoMsg)msg);      
    } else if (msg instanceof CalibrationMsg) {
      calibrationReceived(dest_addr,(CalibrationMsg)msg);
    } else {
      //throw new RuntimeException("messageReceived: Got bad message type: "+msg);
    }
  }


    /**
     * Creates a new instance of SerialReceiver
     */
    public MoteIF getMote() {
        return mote;
    }

    public SerialReceiver() {
        // OK, connect to the serial forwarder and start receiving data
		mote = new MoteIF(PrintStreamMessenger.err, Config.GROUP_ID);
		mote.registerListener(new NodeInfoMsg(), this);
		mote.registerListener(new CalibrationMsg(), this);
        nodeInfoList = new LinkedList<NodeInfoMsg>();
        calibrationList = new LinkedList<CalibrationMsg>();

    }

    public void setMap(MapContainerPanel Map) {
        M=Map;
        M.log("SerialReceiver Map Set for log");
    }

    public void nodeInfoReceived(int dest,NodeInfoMsg msg) {
        M.log(msg.toString());
        RP.ping(msg.get_idA());
        //if (!tooDifferent(msg.get_timestampA(),msg.get_timestampN())) {
            nodeInfoList.addFirst(msg);
        //}
    }

    public void calibrationReceived(int dest,CalibrationMsg msg) {
        M.log(msg.toString());
        calibrationList.addFirst(msg);
    }

    //this return the list of received calibration messages (cannot be changed)
    public LinkedList<CalibrationMsg> getCalibrationList() {
        return calibrationList;
    }

    //this return the list of received nodeinfo messages (cannot be changed)
    public LinkedList<NodeInfoMsg> getNodeInfoList() {
        return nodeInfoList;
    }
    
    public void setRcv(RcvPanel rp) {
        RP = rp;
    }

    //this function check if 2 timestamps that should be roughly the sames are not too different.
    //because if they are too different we can just drop the message.
   /* public boolean tooDifferent(long t1,long t2) {
        if (Math.abs(t1-t2) > Config.MAX_DIFF) {
            return true;
        }
        return false;
    }*/

}
