package net.tinyos.tools;

import java.io.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;
import net.tinyos.message.*;
import java.awt.*;
import java.util.*;
import java.text.*;

public class Monitor {
    static int nbPacket=0;

    public static double convert(byte b) {
        double res=(double)b;
        if (res<0) res=256+res;
        return res;
    }


    public static void main(String args[]) throws IOException {
        if (args.length != 1) {
            System.err.println("usage: java net.tinyos.tools.Monitor <mode>  | <mode> = 1 -> display <mode>= 2 -> log <mode>= 3 -> both");
            System.exit(2);
        }

		int _mode;
		try {
			_mode = Integer.parseInt(args[0]);
		} catch (NumberFormatException e) {
			System.err.println("<mode> must be 1,2 or 3 using 3 by default");
			_mode = 3;
		}


        DecimalFormat f = new DecimalFormat();
        f.setMaximumFractionDigits(2);

        DecimalFormat fent = new DecimalFormat();
        fent.setMaximumFractionDigits(0);

        long startMillis = System.currentTimeMillis();
        long dayCurrentN=Math.round(System.currentTimeMillis()/86400000);
        long dayCurrentR=dayCurrentN;

		PrintWriter fileR=null,fileN=null;

		if (_mode==2 || _mode==3)
		{
			fileR=new PrintWriter(new BufferedWriter
						  (new FileWriter("readings" + startMillis+"_"+dayCurrentR+".txt")));

			fileN=new PrintWriter(new BufferedWriter
						  (new FileWriter("neighbors" + startMillis +"_"+dayCurrentN+".txt")));
		}

        PacketSource reader = BuildSource.makePacketSource();
        if (reader == null) {
            System.err.println("Invalid packet source (check your MOTECOM environment variable)");
            System.exit(2);
        }
        try {
          reader.open(PrintStreamMessenger.err);



          for (;;) {
            byte[] packet = reader.readPacket();
          	Dump.printPacket(System.out, packet);
            System.out.println("");

            if (packet.length == 34 && packet[10]==33) {
				//paquet de readings

				double nodeid = convert(packet[14]);

                double seqno = ( (convert(packet[21])*256*256*256) + (convert(packet[20])*256*256) + (convert(packet[19])*256) + (convert(packet[18])) );

                double tempInt2 = (double) ((((convert(packet[23]) * 256) + convert(packet[22]))/4096*1.5)-0.986)/0.00355;

                double volt = (double) (convert(packet[25]) * 256 + convert(packet[24])) / 4096.0 * 3.0;

				double temp = (double) ((convert(packet[29]) * 256.0 + convert(packet[28])) / 100.0) - 39.6;

                double rawHum = (double) (convert(packet[27]) * 256.0 + convert(packet[26]));
                double hum1=-4;
                double hum2=rawHum*0.0405;
                double hum22=Math.pow(rawHum, 2);
                double hum3=( -2.8 * 0.000001 * hum22);
                double hum = hum1+hum2+hum3;
                double humCorr = (temp - 25) * (0.01 + 0.00008 * rawHum) + hum;

                double PAR = (double) (convert(packet[31]) * 256 + convert(packet[30]));
                //PAR *= volt;

				double TSR = convert(packet[33]) * 256 + convert(packet[32]);


                if (_mode == 1 || _mode == 3) {
					System.out.println("READING:*********");
					System.out.println("node id = " + fent.format(nodeid));
			        System.out.println("seq no = " + fent.format(seqno));
					System.out.println("Itemp = " + f.format(tempInt2));
					System.out.println("volt = " + f.format(volt));
					System.out.println("Temp = " + f.format(temp));
					System.out.println("Humi = " + f.format(humCorr));
					System.out.println("PAR = " + f.format(PAR));
					System.out.println("TSR = " + f.format(TSR));
					System.out.println("*****************");
				}

				if (_mode == 2 || _mode == 3) {
					long dayCurrentNewR=Math.round(System.currentTimeMillis()/86400000);

					try {
						if (dayCurrentNewR>dayCurrentR) {
							fileR.close();
							fileR=new PrintWriter(new BufferedWriter
												 (new FileWriter("readings"+startMillis+"_"+dayCurrentNewR+".txt")));

							dayCurrentR=dayCurrentNewR;
						}


						long currentTime=System.currentTimeMillis();
						//READINGS FORMAT :
						 //TIMESTAMP NODE_ID SEQ_NO ITEMP IVOLT HUMI TEMP PAR TSR
						fileR.println(currentTime +
									 " " + fent.format(nodeid) +
									 " " + fent.format(seqno) +
									 " " + f.format(tempInt2) +
									 " " + f.format(volt) +
									 " " + f.format(humCorr) +
									 " " + f.format(temp) +
									 " " + f.format(PAR) +
									 " " + f.format(TSR));
						fileR.flush();

					} catch (IOException exc) {
				    	System.err.println("Error on fileR : " + exc);
				    }
				}



                //System.out.println("packet reading");
            } else if (packet.length == 30 && packet[10]==34 ) {
				//paquet de neighbors


				double realcount = (convert(packet[23]));

				for (int x=0;x<realcount;x++) {

					double rseqno = ( (convert(packet[21])*256*256*256) + (convert(packet[20])*256*256) + (convert(packet[19])*256) + (convert(packet[18])) );
					double idx =  ( convert(packet[22]));
					double nodeid = ( convert(packet[14]));
					double srcnodeid = convert(packet[24+x]);
					double lqi = convert(packet[26+x]);
					double rssi = (packet[28+x]+128); //convert -> signed

					if (_mode == 1 || _mode == 3) {
						System.out.println("NEIGHBOR:########");
						System.out.println("r seq no = " + fent.format(rseqno));
						System.out.println("idx = " + fent.format(idx));
						System.out.println("node id = " + fent.format(nodeid));
						System.out.println("src node id = " + fent.format(srcnodeid));
						System.out.println("lqi = " + f.format(lqi));
						System.out.println("rssi = " + f.format(rssi));
						System.out.println("#################");
					}

					if (_mode == 2 || _mode == 3) {

						try {

							long dayCurrentNewN=Math.round(System.currentTimeMillis()/86400000);

							if (dayCurrentNewN>dayCurrentN) {
								fileN.close();
								fileN=new PrintWriter(new BufferedWriter
													 (new FileWriter("neighbors"+startMillis+"_"+dayCurrentNewN+".txt")));

								dayCurrentN=dayCurrentNewN;
							}


							long currentTime=System.currentTimeMillis();
							//NEIGHBORS
							//TIMESTAMP IDX R_SEQ_NO NODE_ID SRC_NODE_ID LQI RSSI
							fileN.println(currentTime +
										 " " + fent.format(idx) +
										 " " + fent.format(rseqno) +
										 " " + fent.format(nodeid) +
										 " " + fent.format(srcnodeid) +
										 " " + f.format(lqi) +
										 " " + f.format(rssi));
							fileN.flush();

						} catch (IOException exc) {
							System.err.println("Error on fileR : " + exc);
						}
					}
				}
			}

        /*
        	else
            {
                int numSens = packet[11];

                double temp = (double) (convert(packet[20]) * 256.0 +
                                        convert(packet[21])) / 100.0 - 39.6;
                sensValues[numSens][0]=temp;
                //System.out.println(temp);
                double rawHum = convert(packet[18]) * 256.0 +
                                convert(packet[19]);
                double hum1=-4;
                double hum2=rawHum*0.0405;
                double hum22=Math.pow(rawHum, 2);
                double hum3=( -2.8 * 0.000001 * hum22);
                double hum = hum1+hum2+hum3;
                double humCorr = (temp - 25) * (0.01 + 0.00008 * rawHum) + hum;
                sensValues[numSens][1]=humCorr;

                double volt = (double) (convert(packet[14]) * 256 +
                                        convert(packet[15])) / 4096.0 * 3.0;
                sensValues[numSens][5]=volt;
                double tempInt = 0.00355 * volt + 0.986;
                double tempInt2 = ((convert(packet[16]) * 256 +
                                   convert(packet[17]))/4096*1.5-0.986)/0.00355;
                sensValues[numSens][4]=tempInt2;

                double PAR = convert(packet[22]) * 256 + convert(packet[23]);
                sensValues[numSens][2]=PAR*volt;
                double TSR = convert(packet[24]) * 256 + convert(packet[25]);
                sensValues[numSens][3]=TSR;
                long dayCurrentNew=Math.round(System.currentTimeMillis()/86400000);

                if (dayCurrentNew>dayCurrent) {
                    file.close();
                    file=new PrintWriter(new BufferedWriter
                                         (new FileWriter("output.txt"+dayCurrent)));

                    dayCurrent=dayCurrentNew;
                }
                long currentTime=System.currentTimeMillis();
                file.println(numSens + " " + currentTime +
                             " " + f.format(temp)+ " "+f.format(humCorr)+" "+f.format(TSR)+" "+f.format(PAR)+" "+f.format(volt)+" "+f.format(tempInt2));
                file.flush();

                //System.out.println(tempInt + " " + tempInt2);
                appInter.setAttr(humCorr, 1, numSens);
                appInter.setAttr(temp, 0, numSens);
                appInter.setAttr(TSR, 2, numSens);
                appInter.setAttr(PAR, 3, numSens);
                appInter.setAttr(tempInt2, 4, numSens);
                appInter.setAttr(volt, 5, numSens);
                appInter.incNbPacket(numSens);
                appInter.refresh();
                System.out.flush();
                PrintWriter file2=new PrintWriter(new BufferedWriter
                                     (new FileWriter("current_output.txt",false)));
                file2.println("Last measurement at: "+currentTime+" on sensor n "+numSens);
                for (int k=1; k<=NB_SENS; k++) {
                    file2.println("Sensor n  "+k);
                    file2.println("\tTemperature: "+sensValues[k-1][0]);
                    file2.println("\tHumidity: "+sensValues[k-1][1]);
                    file2.println("\tTSR: "+sensValues[k-1][2]);
                    file2.println("\tPAR: "+sensValues[k-1][3]);
                    file2.println("\tInternal temp"+sensValues[k-1][4]);
                    file2.println("\tVoltage: "+sensValues[k-1][5]);
                    file2.println();
                }
                file2.close();
            }*/
          }
        }
        catch (Exception e) {
            System.err.println("Error on : " + e + ": " + e.getMessage());
        }
    }
}
